@@ -1,14 +1,23 @@
 use std::{mem, ptr};
-use std::alloc::{alloc, dealloc, Layout, realloc};
+use std::alloc::{alloc, dealloc, Layout, realloc, handle_alloc_error};
 use std::any::TypeId;
-use std::cmp::max;
 use std::mem::{MaybeUninit};
-use std::ptr::{null_mut};
+use std::ptr::{NonNull, null_mut};
 
-const MIN_CAPACITY: usize = 2;
+// This is faster then ptr::copy_nonoverlapping,
+// when count is runtime value, and count is small.
+#[inline]
+unsafe fn copy_bytes(src: *const u8, dst: *mut u8, count: usize){
+    // MIRI hack
+    if cfg!(miri) {
+        ptr::copy_nonoverlapping(src, dst, count);
+        return;
+    }
 
-// Never touched
-static mut ZST_ARRAY: [u8;1] = [0];
+    for i in 0..count{
+        *dst.add(i) = *src.add(i);
+    }
+}
 
 /// Type erased vec-like container.
 /// All elements have the same type.
@@ -17,76 +26,91 @@ static mut ZST_ARRAY: [u8;1] = [0];
 ///
 /// *`T:'static` due to TypeId requirements*
 pub struct AnyVec {
-    mem: *mut u8,
+    mem: NonNull<u8>,
     capacity: usize,        // in elements
     len: usize,             // in elements
     element_layout: Layout, // size is aligned
     type_id: TypeId,        // purely for safety checks
-    drop_fn: fn(ptr: *mut u8, len: usize)
+    drop_fn: Option<fn(ptr: *mut u8, len: usize)>
 }
 
 impl AnyVec {
     pub fn new<T:'static>() -> Self {
-        Self::with_capacity::<T>(MIN_CAPACITY)
+        Self::with_capacity::<T>(0)
     }
 
-    pub fn with_capacity<T:'static>(mut capacity: usize) -> Self {
-        capacity = max(capacity, MIN_CAPACITY);
-
-        let element_layout = Layout::new::<T>();
-        let mem = unsafe {
-            if element_layout.size() != 0 {
-                alloc(Layout::from_size_align_unchecked(
-                    element_layout.size() * capacity, element_layout.align()
-                ))
-            } else {
-                &mut ZST_ARRAY as *mut u8
-            }
-        };
-        Self{
-            mem,
-            capacity,
+    pub fn with_capacity<T:'static>(capacity: usize) -> Self {
+        let mut this = Self{
+            mem: NonNull::<u8>::dangling(),
+            capacity: 0,
             len: 0,
-            element_layout,
+            element_layout: Layout::new::<T>(),
             type_id: TypeId::of::<T>(),
-            drop_fn: |mut ptr: *mut u8, len: usize|{
-                // compile time check
+            drop_fn:
                 if !mem::needs_drop::<T>(){
-                    return;
+                    None
+                } else{
+                    Some(|mut ptr: *mut u8, len: usize|{
+                        for _ in 0..len{
+                            unsafe{
+                                ptr::drop_in_place(ptr as *mut T);
+                                ptr = ptr.add(mem::size_of::<T>());
+                            }
+                        }
+                    })
                 }
-
-                for _ in 0..len{
-                    unsafe{
-                        ptr::drop_in_place(ptr as *mut T);
-                        ptr = ptr.add(mem::size_of::<T>());
-                    }
-                }
-            }
-        }
+        };
+        this.set_capacity(capacity);
+        this
     }
 
-    fn set_capacity(&mut self, mut new_capacity: usize){
+    fn set_capacity(&mut self, new_capacity: usize){
         // Never cut
         debug_assert!(self.len <= new_capacity);
 
-        new_capacity = max(MIN_CAPACITY, new_capacity);
-        unsafe{
-            if self.element_layout.size() != 0 {
+        if self.capacity == new_capacity {
+            return;
+        }
+
+        if self.element_layout.size() != 0 {
+            unsafe{
                 let mem_layout = Layout::from_size_align_unchecked(
                     self.element_layout.size() * self.capacity, self.element_layout.align()
                 );
-                // mul carefully, to prevent overflow.
-                let new_mem_size = self.element_layout.size().checked_mul(new_capacity).unwrap();
-                self.mem = realloc(self.mem, mem_layout,new_mem_size);
+
+                self.mem =
+                    if new_capacity == 0 {
+                        dealloc(self.mem.as_ptr(), mem_layout);
+                        NonNull::<u8>::dangling()
+                    } else {
+                        // mul carefully, to prevent overflow.
+                        let new_mem_size = self.element_layout.size().checked_mul(new_capacity).unwrap();
+                        let new_mem_layout = Layout::from_size_align_unchecked(
+                            new_mem_size, self.element_layout.align()
+                        );
+
+                        if self.capacity == 0 {
+                            // allocate
+                            NonNull::new(alloc(new_mem_layout))
+                        } else {
+                            // reallocate
+                            NonNull::new(realloc(
+                                self.mem.as_ptr(), mem_layout,new_mem_size
+                            ))
+                        }
+                        .unwrap_or_else(|| handle_alloc_error(new_mem_layout))
+                    }
             }
-            self.capacity = new_capacity;
         }
+        self.capacity = new_capacity;
     }
 
     #[cold]
     #[inline(never)]
     fn grow(&mut self){
-        self.set_capacity(self.capacity * 2);
+        self.set_capacity(
+             if self.capacity == 0 {2} else {self.capacity * 2}
+        );
     }
 
     /// Pushes one element without actually writing anything.
@@ -104,7 +128,7 @@ impl AnyVec {
             self.grow();
         }
 
-        let new_element = self.mem.add(self.element_layout.size() * self.len);
+        let new_element = self.mem.as_ptr().add(self.element_layout.size() * self.len);
         self.len += 1;
 
         std::slice::from_raw_parts_mut(
@@ -138,33 +162,44 @@ impl AnyVec {
         }
     }
 
+    #[inline]
+    fn drop_element(&mut self, ptr: *mut u8, len: usize){
+        if let Some(drop_fn) = self.drop_fn{
+            (drop_fn)(ptr, len);
+        }
+    }
+
     /// drop element, if out is null.
     #[inline]
     unsafe fn swap_take_bytes_impl(&mut self, index: usize, out: *mut u8) {
         assert!(index < self.len);
 
         // 1. drop element at index
-        let element = self.mem.add(self.element_layout.size() * index);
+        let element = self.mem.as_ptr().add(self.element_layout.size() * index);
         if !out.is_null() {
             ptr::copy_nonoverlapping(element, out, self.element_layout.size());
         } else {
-            (self.drop_fn)(element, 1);
+            self.drop_element(element, 1);
         }
 
         // 2. move element
         let last_index = self.len - 1;
         if index != last_index {
-            let last_element = self.mem.add(self.element_layout.size() * last_index);
-            ptr::copy_nonoverlapping(last_element, element, self.element_layout.size());
+            let last_element = self.mem.as_ptr().add(self.element_layout.size() * last_index);
+            //ptr::copy_nonoverlapping(last_element, element, self.element_layout.size());
+            copy_bytes(last_element, element, self.element_layout.size());
         }
 
         // 3. shrink len
         self.len -= 1;
     }
 
+    /// Type erased version of [`Vec::swap_remove`]. Due to this, does not return element.
+    ///
     /// # Panics
     ///
-    /// Panics if index out of bounds.
+    /// Panics if index is out of bounds.
+    #[inline]
     pub fn swap_remove(&mut self, index: usize) {
         unsafe{
             self.swap_take_bytes_impl(index, null_mut());
@@ -181,6 +216,7 @@ impl AnyVec {
     /// * Panics if out len does not match element size.
     ///
     /// [`swap_remove`]: Self::swap_remove
+    #[inline]
     pub unsafe fn swap_take_bytes_into(&mut self, index: usize, out: &mut[u8]){
         assert_eq!(out.len(), self.element_layout.size());
         self.swap_take_bytes_impl(index, out.as_mut_ptr());
@@ -193,6 +229,7 @@ impl AnyVec {
     /// # Panics
     /// Panics if index out of bounds.
     /// Panics if type mismatch.
+    #[inline]
     pub fn swap_take<T:'static>(&mut self, index: usize) -> T {
         assert_eq!(TypeId::of::<T>(), self.type_id);
 
@@ -205,14 +242,19 @@ impl AnyVec {
 
     #[inline]
     pub fn clear(&mut self){
-        (self.drop_fn)(self.mem, self.len);
+        let len = self.len;
+
+        // Prematurely set the length to zero so that even if dropping the values panics users
+        // won't be able to access the dropped values.
         self.len = 0;
+
+        self.drop_element(self.mem.as_ptr(), len);
     }
 
     #[inline]
     pub unsafe fn as_slice_unchecked<T>(&self) -> &[T]{
         std::slice::from_raw_parts(
-            self.mem as *const T,
+            self.mem.as_ptr().cast::<T>(),
             self.len,
         )
     }
@@ -229,9 +271,9 @@ impl AnyVec {
     }
 
     #[inline]
-    pub unsafe fn as_mut_slice_unchecked<T:'static>(&self) -> &mut[T]{
+    pub unsafe fn as_mut_slice_unchecked<T:'static>(&mut self) -> &mut[T]{
         std::slice::from_raw_parts_mut(
-            self.mem as *mut T,
+            self.mem.as_ptr().cast::<T>(),
             self.len,
         )
     }
@@ -240,7 +282,7 @@ impl AnyVec {
     ///
     /// Panics if type mismatch.
     #[inline]
-    pub fn as_mut_slice<T:'static>(&self) -> &mut[T]{
+    pub fn as_mut_slice<T:'static>(&mut self) -> &mut[T]{
         assert_eq!(TypeId::of::<T>(), self.type_id);
         unsafe{
             self.as_mut_slice_unchecked::<T>()
@@ -248,19 +290,23 @@ impl AnyVec {
     }
 
     /// Element TypeId
+    #[inline]
     pub fn element_typeid(&self) -> TypeId{
         self.type_id
     }
 
     /// Element Layout
+    #[inline]
     pub fn element_layout(&self) -> Layout {
         self.element_layout
     }
 
+    #[inline]
     pub fn len(&self) -> usize {
         self.len
     }
 
+    #[inline]
     pub fn capacity(&self) -> usize {
         self.capacity
     }
@@ -269,12 +315,7 @@ impl AnyVec {
 impl Drop for AnyVec {
     fn drop(&mut self) {
         self.clear();
-        if self.element_layout.size() != 0 {
-            unsafe{
-                dealloc(self.mem, Layout::from_size_align_unchecked(
-                    self.element_layout.size() * self.capacity, self.element_layout.align()))
-            }
-        }
+        self.set_capacity(0);
     }
 }
 
@@ -382,14 +423,18 @@ mod tests {
             ]);
         }
 
-        {
-            raw_vec.swap_remove(2);
-            assert_equal(raw_vec.as_slice::<String>(), &[
-                String::from("0"),
-                String::from("4"),
-                String::from("3"),
-            ]);
-        }
+        raw_vec.swap_remove(2);
+        assert_equal(raw_vec.as_slice::<String>(), &[
+            String::from("0"),
+            String::from("4"),
+            String::from("3"),
+        ]);
+
+        raw_vec.swap_remove(2);
+        assert_equal(raw_vec.as_slice::<String>(), &[
+            String::from("0"),
+            String::from("4"),
+        ]);
     }
 
     #[test]
@@ -418,4 +463,4 @@ mod tests {
             String::from("3"),
         ]);
     }
-}
\ No newline at end of file
+}
